//
//  Logging.cpp
//  HTTPserver
//
//  Created by Lyle Moffitt on 5/9/14.
//  Copyright (c) 2014 Lyle Moffitt. All rights reserved.
//

#include "Logging.h"
// #include "sighandler.h"

//#include <boost/core/null_deleter.hpp>


//#include <boost/date_time.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/date_time/posix_time/posix_time_types.hpp>

#include <boost/format.hpp>

#include <boost/lambda/lambda.hpp>

//#include <boost/log/core.hpp>

//#include <boost/log/keywords/>

#include <boost/log/attributes/clock.hpp>
#include <boost/log/attributes/current_process_id.hpp>
#include <boost/log/attributes/named_scope.hpp>
#include <boost/log/attributes/current_thread_id.hpp>

#include <boost/log/expressions.hpp>
#include <boost/log/expressions/formatters/xml_decorator.hpp>
#include <boost/log/expressions/formatters/named_scope.hpp>
#include <boost/log/expressions/formatters/format.hpp>
#include <boost/log/expressions/formatters/stream.hpp>
#include <boost/log/expressions/formatters/date_time.hpp>

//#include <boost/log/sinks.hpp>
#include <boost/log/sinks/async_frontend.hpp>
#include <boost/log/sinks/text_ostream_backend.hpp>

#include <boost/log/sources/global_logger_storage.hpp>
#include <boost/log/sources/severity_logger.hpp>

#include <boost/log/support/date_time.hpp>

#include <boost/log/trivial.hpp>

//#include <boost/log/utility/setup/common_attributes.hpp>

#include <boost/phoenix/bind.hpp>
#include <boost/phoenix/operator.hpp>

#include <boost/smart_ptr/shared_ptr.hpp>

#include <iostream>
#include <regex>
#include <string>


//namespace Log {

// Adjust b/c empty_deleter was moved in 1.55
#if (BOOST_VERSION / 100 % 1000) > 55
#   include <boost/utility/empty_deleter.hpp>
#   define no_op_deleter boost::empty_deleter
#   define LOG_SCOPE_FILE_FORMAT "%F : %n"
#   define LOG_SCOPE_TERM_FORMAT "%F : %n"
#else 
#   include <boost/log/utility/empty_deleter.hpp>
#   define no_op_deleter logging::empty_deleter
#   define LOG_SCOPE_FILE_FORMAT "%f : %n"
#   define LOG_SCOPE_TERM_FORMAT "%f\n%n"
#endif

#ifndef LOG_FILE_PATH
#   ifdef TESTING
#       define LOG_FILE_PATH "./"
#   else
#       define LOG_FILE_PATH "/var/www/html/cups_driver/"
#   endif
#endif

#ifndef LOG_FILE_NAME_FMT
#   define LOG_FILE_NAME_FMT   \
        LOG_FILE_PATH "%Y%m%d_%H%M%S_%5N.html"
#endif


/// Defines the log file preamble -- specifies an HTML file with sortable table.
#ifndef LOG_FILE_HEADER
#   define LOG_FILE_HEADER     \
"<!DOCTYPE html><HTML>"\
"\n<HEAD>"\
"\n  <font face=\"monospace\">"\
"\n  <TITLE>SORTED table</TITLE>"\
"\n  <SCRIPT src=\"http://www.kryogenix.org/code/browser/sorttable/sorttable.js\""\
"\n         type=\"text/javascript\" charset=\"utf-8\">"\
"\n  </SCRIPT>"\
"\n</HEAD>"\
"\n<BODY>"\
"\n  <table border=\"0\" cellspacing=\"5\" cellpadding=\"5\" class=\"sortable\">"\
"\n    <thead>"\
"\n      <tr> <th>%1%</th> <th>%2$06d</th> <th>%3%</th> <th>%4%</th> <th>%5%</th> <th>%6%</th> </tr>"\
"\n    </thead>"\
"\n    <tbody>"
#endif

/// Defines the format of every row in the table
#ifndef LOG_FILE_ROW_FMT
#   define LOG_FILE_ROW_FMT    \
"\n      <tr> "\
"\n        <td>%1%</td> <td>%2%</td> <td>%3%</td> <td>%4%</td>"\
"\n        <td><pre>%5%</pre></td>"\
"\n        <td><pre>%6%</pre></td>"\
"\n      </tr>"
#endif

/// Defines the log file close
#ifndef LOG_FILE_FOOTER
#   define LOG_FILE_FOOTER     \
"\n    </tbody>"\
"\n  </table>"\
"\n</BODY>"\
"\n</HTML>"
#endif

#ifndef LOG_FILE_POOL_SIZE
#   define LOG_FILE_POOL_SIZE (8 * 1024) // 8 Kib
#endif

#ifndef LOG_FILE_ENABLE
#   define LOG_FILE_ENABLE
#endif
#ifndef LOG_FILE_LEVEL
#   define LOG_FILE_LEVEL trace
#endif
#ifndef LOG_FILE_SCOPE
#   define LOG_FILE_SCOPE ".**"
#endif


#ifndef LOG_TERM_ENABLE
#   define LOG_TERM_ENABLE
#endif
#ifndef LOG_TERM_LEVEL
#   define LOG_TERM_LEVEL warning
#endif
#ifndef LOG_TERM_SCOPE
#   define LOG_TERM_SCOPE ".**"
#endif


file_sink_ptr   log_data::file_sink;

text_sink_ptr    log_data::term_sink;

std::shared_ptr<filter_config>   log_data::file_filter_config;

std::shared_ptr<filter_config>   log_data::term_filter_config;

LVL
lvl_from_string(const std::string & _s)
{

    auto match_str = [&](LVL lvl){
        return (boost::format("%1%|%2%")
                % logging::trivial::to_string( lvl )
                % std::to_string((int)( lvl ))).str();
    };

    auto has_lvl = [&](LVL lvl){
        return std::regex_search( _s ,std::regex(match_str(lvl),
                                                 std::regex::icase));
    };

    if( has_lvl( LVL::trace  ) ) return LVL::trace   ;
    if( has_lvl( LVL::debug  ) ) return LVL::debug   ;
    if( has_lvl( LVL::info   ) ) return LVL::info    ;
    if( has_lvl( LVL::warning) ) return LVL::warning ;
    if( has_lvl( LVL::error  ) ) return LVL::error   ;
    if( has_lvl( LVL::fatal  ) ) return LVL::fatal   ;

    LOG(LVL::error)
    <<"";

    throw std::invalid_argument("");
}

bool generic_filter
(   logging::value_ref< logging::trivial::severity_level >  const&  level
 ,  logging::value_ref< attrs::named_scope_list >           const&  scope
 ,  filter_config                                           const   conf
 ){
    if(!( level >= conf.level ))
        return false;

    const std::regex e (conf.scope);
    for( auto & i : scope.get() ){
        if( (std::regex_search(i.file_name.c_str(), e)) )
            return true;
        if( (std::regex_search(i.scope_name.c_str(), e)) )
            return true;
    }
    return false;
}

bool file_filter
(   logging::value_ref< logging::trivial::severity_level >  const&  level
 ,  logging::value_ref< attrs::named_scope_list >           const&  scope
 ){
    return generic_filter(level, scope, *log_data::file_filter_config);
}


bool term_filter
(   logging::value_ref< logging::trivial::severity_level >  const&  level
 ,  logging::value_ref< attrs::named_scope_list >           const&  scope
 ){
    return generic_filter(level, scope, *log_data::term_filter_config);
}

    //global logger definition
BOOST_LOG_GLOBAL_LOGGER_INIT( logger_name, global_logger_type )
{

    // Create the logging device that will be made global
    // Possible levels = trace,debug,info,warning,error,fatal
    global_logger_type lg;
    
//    logging::add_common_attributes(); //unnecessary given init below
    /* Imbues the log with the following attributes:
     *      LineID      - logging records counter 
     *      TimeStamp   - local posix_time generator 
     *      ProcessID   - current process identifier 
     *      ThreadID    - current thread identifier
     */

    logging::core_ptr core = logging::core::get();
    core->add_global_attribute("TimeStamp",    attrs::local_clock()        );
    core->add_global_attribute("ProcessID",    attrs::current_process_id() );
    core->add_global_attribute("ThreadID",     attrs::current_thread_id()  );
    core->add_global_attribute("Scope",        attrs::named_scope()        );


    //Initialize other logging outputs
    log_data::init_file();
    log_data::init_term();


    // Finish initialization of logger
    return lg;
}

void
log_data::init_file()
{
#ifdef LOG_FILE_ENABLE
    do{
        // Create a text file sink (data storage backend)
        log_data::file_sink.reset(new file_sink_type
        (   keywords::file_name        = LOG_FILE_NAME_FMT  //filename template
         ,  keywords::rotation_size    = LOG_FILE_POOL_SIZE //capped max file zise
         ,  keywords::auto_flush       = true               //possibly flush after write
//         ,  keywords::time_based_rotation = sinks::file::rotation_at_time_point(12, 0, 0) //rotate at noon
//         ,  keywords::time_based_rotation = sinks::file::rotation_at_time_interval(posix_time::hours(6)) //rotate every 6 hours
         ));

        //http://www.boost.org/doc/libs/1_59_0/libs/log/doc/html/log/detailed/expressions.html#log.detailed.expressions.formatters.date_time
        auto fmtTimeStamp   =
            expr::format_date_time<boost::posix_time::ptime>("TimeStamp", "%Y-%m-%d %H:%M:%S.%f");
        auto fmtProcessId   =
            expr::attr<attrs::current_process_id::value_type>("ProcessID");
        auto fmtThreadId    =
            expr::attr<attrs::current_thread_id::value_type>("ThreadID");
        auto fmtSeverity    =
            expr::attr<logging::trivial::severity_level>("Severity");
        //http://www.boost.org/doc/libs/1_59_0/libs/log/doc/html/log/detailed/expressions.html#log.detailed.expressions.formatters.named_scope
        auto fmtScope       =
            expr::format_named_scope
            ("Scope"
             ,keywords::format              = LOG_SCOPE_FILE_FORMAT
             ,keywords::iteration           = expr::reverse // or 'forward'
             ,keywords::depth               = 0
             ,keywords::delimiter           = "\n|>  "      //default = "->"
             //Not supported in 1.55
//             ,keywords::incomplete_marker   = ""            //default = "..."
//             ,keywords::empty_marker        = "[[ N/A ]]"   //default = ""
             );

        //set sink format
        logging::formatter logFmt = expr::format( LOG_FILE_ROW_FMT )
        %   fmtTimeStamp
        %   fmtProcessId
        %   fmtThreadId
        %   fmtSeverity
        %   expr::xml_decor[ (expr::stream << fmtScope)   ]
        %   expr::xml_decor[ (expr::stream << expr::smessage) ]
        ;
        log_data::file_sink->set_formatter(logFmt);

        //set sink default filer
        log_data::file_filter_config          = std::make_shared<filter_config>();
        log_data::file_filter_config->level   = logging::trivial::LOG_FILE_LEVEL;
        log_data::file_filter_config->scope   = LOG_FILE_SCOPE;

        log_data::file_sink->set_filter
        (boost::phoenix::bind
         (&file_filter
          ,expr::attr< logging::trivial::severity_level >("Severity").or_none()
          ,expr::attr< attrs::named_scope_list >("Scope").or_none()
          )
         );

        // Set header and footer writing functions
        log_data::file_sink->locked_backend()->set_open_handler
        (
         boost::lambda::_1 << boost::format( LOG_FILE_HEADER )
          % "TimeStamp" % "ProcessID" % "ThreadID" % "Severity" % "Scope" % "Message"
         );
        log_data::file_sink->locked_backend()->set_close_handler
        (
         boost::lambda::_1 << LOG_FILE_FOOTER
         );

        // Add the sink to the core logging utility
        logging::core::get()->add_sink(log_data::file_sink);
    }while(0);

#endif //LOG_FILE_ENABLE
}

void
log_data::init_term()
{
#ifdef LOG_TERM_ENABLE
    do{
        // create sink
        log_data::term_sink = boost::make_shared<text_sink_type>();

        // hoook sink backen to stderr(clog)
        log_data::term_sink->locked_backend()->add_stream
        (
            boost::shared_ptr<std::ostream>(
                                            & std::clog
                                            , no_op_deleter()
                                            )
         );

        // set auto-flush for text-backend
        log_data::term_sink->locked_backend()->auto_flush(true);

        //define sink format
        auto fmtSeverity    =
            expr::attr<logging::trivial::severity_level>("Severity");
        auto fmtScope       =
            expr::format_named_scope
            ("Scope"
             ,keywords::format              = LOG_SCOPE_TERM_FORMAT
             ,keywords::iteration           = expr::reverse // or 'forward'
             ,keywords::depth               = 1             //default = 0
//             ,keywords::delimiter           = "\n|>\t"      //default = "->"
             //Not supported on boost 1.55
//             ,keywords::incomplete_marker   = ""            //default = "..."
//             ,keywords::empty_marker        = ""            //default = ""
             ) ;


        //set sink format
        logging::formatter txtFmt = expr::format("%1%\n\t[%2%] %3%\n")
        %   fmtScope
        %   fmtSeverity
        %   expr::smessage
        ;
        log_data::term_sink->set_formatter(txtFmt);

        // set sink filter
        log_data::term_filter_config          = std::make_shared<filter_config>();
        log_data::term_filter_config->level   = logging::trivial::LOG_TERM_LEVEL;
        log_data::term_filter_config->scope   = LOG_TERM_SCOPE;

        log_data::term_sink->set_filter
        (boost::phoenix::bind
         (&term_filter
          ,expr::attr< logging::trivial::severity_level >("Severity").or_none()
          ,expr::attr< attrs::named_scope_list >("Scope").or_none()
          )
         );

        // register sink
        logging::core::get()->add_sink(log_data::term_sink);
    }while(0);
#endif //LOG_TERM_ENABLE
}





//}
