//
//  Logging.h
//  HTTPserver
//
//  Created by Lyle Moffitt on 5/9/14.
//  Copyright (c) 2014 Lyle Moffitt. All rights reserved.
//

#ifndef __HTTPserver__Logging__
#define __HTTPserver__Logging__

#define BOOST_LOG_DYN_LINK 1 //Required to dynamically link boost-log libraries


#include <boost/log/trivial.hpp>

//#include <boost/log/sinks.hpp>
#include <boost/log/sinks/async_frontend.hpp>
#include <boost/log/sinks/sync_frontend.hpp>
#include <boost/log/sinks/text_ostream_backend.hpp>
#include <boost/log/sinks/text_file_backend.hpp>

#include <boost/log/attributes/named_scope.hpp>

#include <boost/log/sources/global_logger_storage.hpp>

#include <boost/date_time/posix_time/posix_time.hpp>

//namespace Log {

namespace logging   = boost::log;//::v2s_mt_posix;
namespace attrs     = logging::attributes;
namespace src       = logging::sources;
namespace sinks     = logging::sinks;
namespace expr      = logging::expressions;
namespace keywords  = logging::keywords;

namespace posix_time = boost::posix_time;


#define assert_and_log( _test_ , _msg_ )\
    do{if( !(_test_) ){    \
        BOOST_LOG_FUNCTION();   \
        LOG(LVL::fatal) << "(" #_test_ ") : " _msg_;    \
        abort();  \
    }}while(false)

#define LOG_SCOPE_NAME( _scope_name_ )\
    BOOST_LOG_NAMED_SCOPE( _scope_name_ )

#define LOG_THIS_SCOPE()\
    BOOST_LOG_FUNCTION()

#define LOG( _severity_level_ ) \
    BOOST_LOG_SEV( logger_name::get() , _severity_level_ )


    
typedef logging::trivial::severity_level LVL;

/// Define logger type
typedef src::severity_logger_mt<
    logging::trivial::severity_level    // the type of the severity level
//    ,std::string         // the type of the channel name
> global_logger_type;


/// Define sink type
typedef sinks::asynchronous_sink<
    sinks::text_file_backend
//    ,sinks::unbounded_fifo_queue //default
//    ,sinks::bounded_fifo_queue<100, sinks::drop_on_overflow> //alternative
> file_sink_type;
typedef boost::shared_ptr< file_sink_type > file_sink_ptr;

typedef sinks::synchronous_sink<
    sinks::text_ostream_backend
> text_sink_type;
typedef boost::shared_ptr< text_sink_type > text_sink_ptr;

struct filter_config
{
    LVL                 level;
    std::string         scope;
};

struct log_data
{
static void init_term();
static void init_file();

static file_sink_ptr    file_sink;

static text_sink_ptr    term_sink;



static std::shared_ptr<filter_config>   file_filter_config;

static std::shared_ptr<filter_config>   term_filter_config;
};

LVL
lvl_from_string(const std::string & _s);

bool generic_filter
(   logging::value_ref< logging::trivial::severity_level >  const&  level
 ,  logging::value_ref< attrs::named_scope_list >           const&  scope
 ,  filter_config                                           const   conf
 );

//bool file_filter
//(   logging::value_ref< logging::trivial::severity_level >  const& level
// ,  logging::value_ref< attrs::named_scope_list >           const& scope
// );
//
//bool term_filter
//(   logging::value_ref< logging::trivial::severity_level >  const& level
// ,  logging::value_ref< attrs::named_scope_list >           const& scope
// );

    // Global logger declaration
BOOST_LOG_GLOBAL_LOGGER( logger_name, global_logger_type );




//}





#endif /* defined(__HTTPserver__Logging__) */
